/*
 * author: Darren Bounds <darren@sfhn.net>
 * copyright: Copyright (C) 2002 by Darren Bounds
 * license: This software is under GPL version 2 of license
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * packit official page at http://www.intrusense.com/~dbounds
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "../include/packit.h"
#include "../include/capture.h"
#include "../include/error.h"

#define READ_TIMEOUT 		500
#define DEFAULT_SNAP_LEN       	64


pcap_t *p;
static int packet_c;

void
capture_stats()
{
    struct pcap_stat p_stats;

    pcap_stats(p, &p_stats);

    fprintf(stderr, "------ Packet Capture Statistics ------\n");
    fprintf(stderr,
        "Packet(s) received: %d  "
        "Packet(s) dropped: %d  "
        "Packet(s) processed: %d ",
        p_stats.ps_recv, p_stats.ps_drop, packet_c);
 
    fprintf(stderr, "\n");

    return;
}

void
process_packet_data(unsigned char *user, struct pcap_pkthdr *pkthdr, unsigned char *packet)
{
    char time_stamp[64];
    struct libnet_ethernet_hdr *ehdr;
    struct libnet_ipv4_hdr *iphdr;

    if(display)
    {
        ehdr = (struct libnet_ethernet_hdr *)(packet);

        snprintf(time_stamp, 64, "%02ld:%02ld:%02ld.%06u", 
            (pkthdr->ts.tv_sec % 86400) / 3600, 
            ((pkthdr->ts.tv_sec % 86400) % 3600) / 60, 
            (pkthdr->ts.tv_sec % 86400) % 40, 
            (unsigned)pkthdr->ts.tv_usec); 

        iphdr = (struct libnet_ipv4_hdr *)(packet + hdrlen);

        switch(iphdr->ip_p)
        {
            case IPPROTO_TCP: 
                display_tcp_data(packet, time_stamp, ehdr, iphdr);
                packet_c++;
                break;

            case IPPROTO_UDP: 
                display_udp_data(packet, time_stamp, ehdr, iphdr);
                packet_c++;
                break;

            case IPPROTO_ICMP: 
                display_icmp_data(packet, time_stamp, ehdr, iphdr);
                packet_c++;
                break;
        }

        return;
    }
}

int
start_packet_capture(char *pfilter, short count)
{
    int dlink;
    struct bpf_program bpf;
    bpf_u_int32 localnet, netmask;
    pcap_dumper_t *pd;

    signal(SIGTERM, capture_clean_exit);
    signal(SIGINT, capture_clean_exit);
    signal(SIGQUIT, capture_clean_exit);
    signal(SIGHUP, capture_clean_exit);

    if(strlen(read_filename) > 0)
    {
        if((p = pcap_open_offline(read_filename, errbuf)) == NULL)
        {
            fprintf(stderr, "\nError: Unable to open file: %s\n", errbuf); 
            exit(EXIT_FAILURE);
        }
    }
    else
    {
        if (device == NULL)
           if((device = pcap_lookupdev(errbuf)) == NULL)
           {
               fprintf(stderr, "\nError: Unable to find appropriate device: %s\n", errbuf);
               exit(EXIT_FAILURE);
           }

        if((p = pcap_open_live(device, DEFAULT_SNAP_LEN, 1, READ_TIMEOUT, errbuf)) == NULL)
        {
            fprintf(stderr, "\nError: Unable to open device: %s", errbuf);
            exit(EXIT_FAILURE);
        }
    }

    display = 1;
    pd = NULL;

    if(strlen(write_filename) > 0)
    {
        if((pd = pcap_dump_open(p, write_filename)) == NULL)
        {
            fprintf(stderr, "\nError: Unable to initialize packet capture: %s\n", pcap_geterr(p));
            exit(EXIT_FAILURE);
        }

        display--;
    }

    pcap_lookupnet(device, &localnet, &netmask, errbuf);

    if(pcap_compile(p, &bpf, pfilter, 0, netmask) < 0)
    {
        fprintf(stderr, "\nError: Unable to process packet filters: %s", pcap_geterr(p));
        exit(EXIT_FAILURE);
    }

    if(pcap_setfilter(p, &bpf) < 0)
    {
        fprintf(stderr, "\nError: Unable to set packet filters: %s", pcap_geterr(p));
        exit(EXIT_FAILURE);
    }

    if((dlink = pcap_datalink(p)) < 0)
    {
        fprintf(stderr, "\nError: Unable to determine datalink type: %s", pcap_geterr(p));
        exit(EXIT_FAILURE);
    }

    switch(dlink)
    {
        case DLT_NULL:
            hdrlen = 4;
            break;

        case DLT_EN10MB:
            hdrlen = PACKIT_ETH_H; 
            break;

        default:
            hdrlen = PACKIT_ETH_H; 
            break;
    }

    fprintf(stderr, "Mode: Packet Capture ");

    if(strlen(read_filename) > 0)
        fprintf(stderr, "using file: %s ", read_filename);
    else
        fprintf(stderr, "using device: %s ", device);

    if(pfilter)
        fprintf(stderr, "[%s]", pfilter);

    fprintf(stderr, "\n");

    packet_c = 0;

    if(display)
    {
        if(pcap_loop(p, count, (pcap_handler)process_packet_data, NULL) < 0)
        {
            fprintf(stderr, "\nError: Unable to initialize pcap_loop: %s", pcap_geterr(p));
            exit(EXIT_FAILURE);
        }
    }
    else
    {
        if(pcap_loop(p, count, (pcap_handler)pcap_dump, (unsigned char *)pd) < 0)
        {
            fprintf(stderr, "\nError: Unable to initialize pcap_loop: %s", pcap_geterr(p));
            exit(EXIT_FAILURE);
        }
    }

    capture_clean_exit(1);

    return 1;
}


void
capture_clean_exit(int sig)
{
    fprintf(stderr, "\n\n");

    capture_stats();
    pcap_close(p);

    fprintf(stderr, "\n");

    exit(EXIT_FAILURE);

    return;
}
